@@ -4,10 +4,13 @@
 from __future__ import absolute_import
 from urlparse import urlsplit
 
-class Extension(object):
+class DjangoExtension(object):
     def django_file(self, name=None):
         """
-        Build from response content the django ContentFile instance.
+        Convert content of response into django `ContentFile` object.
+
+        :param name: specify name of file, otherwise the last segment in
+        URL path will be used as filename.
         """
        
         from django.core.files.base import ContentFile
